#! /usr/bin/perl -w

# find words whose Root type field doesn't match the form of 
# the actual word

# first: check for dashes or spaces in supposedly 'r' entries
# and for lack thereof in 'd' entries

# later: check for the full array of vowel types in all non-'d' entries

use strict;


my $debug = 1;
my $inputfile = "nxcgtx.rdb";
my $pickiness = 1;
my $arg;

open IN, $inputfile 	or die "can't open $inputfile for reading\n";
#open OUT, "> $outputfile" 	or die "can't open $outputfile for writing\n";

while ( <IN> )
{

    if ( $. <= 2 ) {    # skip header
	next;
    }

    s/[\r\n]+//;	    # remove CR/LF

    my @fields = split /\t/;
    if ( $fields[1] eq 'r'  &&  $fields[0] !~ m/^[a-z\']+$/ ) {
	print qq(line $.: possible non-root marked as 'r': "$fields[0]"\n);
    } elsif ( $fields[1] eq 'd' && $fields[0] !~ m/[ -]/ ) {
	print qq(line $.: possible non-derived item marked as 'd': "$fields[0]"\n);
    } elsif ( $fields[1] eq 'm' && $fields[0] !~ m/(e|oq)/ ) {
	print qq(line $.: possible non-{jum} item marked as 'm': "$fields[0]"\n);
    } elsif ( $fields[1] eq 's' && $fields[0] !~ m/(a|ox)/ ) {
	print qq(line $.: possible non-suffix item marked as 's': "$fields[0]"\n);
    } elsif ( $fields[1] !~ m/^.$/ )  {
	print qq(line $. appears to be misformatted:\n$_\n);
    } elsif ( $fields[4] && $fields[4] !~ m/^[0-9-]+\??$/ )  {
	print qq(line $. has a bad date: "$fields[4]"\n);
    } elsif ( $fields[3] && $fields[3] =~ m/[0-9]/ )  {
	print qq(line $. may have date in place of or run together with its category:)
		. qq("$fields[3]"\n);
    } elsif ( defined $fields[6] ) {
	print qq(line $. has too many fields:\n$_\n);
    }

    for ( my $i = 0; $i < 6; $i++ ) {
	if ( $fields[$i] && $fields[$i] =~ m/^\s/ && $fields[3] ne "nxix'biq") {
	    if ( $i == 2 && $fields[3] eq "nxix'biq" ) {
		next;
	    }
	    print qq(line $. has leading space in field $i: ") . $fields[$i] . qq("\n);
	}
	if ( $fields[$i] && $fields[$i] =~ m/\s$/ ) {
	    if ( $i == 2 && $pickiness < 3 ) {
		next;
	    }
	    print qq(line $. has trailing space in field $i: ") . $fields[$i] . qq("\n);
	}
    }
    ###TODO check for other types
}


