#! /usr/bin/perl -w

###TODO add a help option and usage print function

# there are lots of magic numbers, literal integer constants that
# should be abstracted out with 'use constant'

use warnings;
use strict;
use POSIX qw(strftime);
use Getopt::Long;

Getopt::Long::Configure('bundling');

# field numbers offset by 1 because we prepend a random number
use constant WORD => 1;
use constant ROOTTYPE => 2;
use constant GLOSS => 3;
use constant DATE => 4;

use constant CREATION_YEAR => 1998;
use constant SECS_PER_DAY  => 86400;
use constant SECS_PER_HOUR => 3600;
# if I was quizzed on a word less than this many days ago, and I knew
# it then, don't quiz on it again now
use constant REQUIZ_INTERVAL => 30;	###TODO unused? implement or remove?

# number of turns to wait before re-quizzing an unknown word
use constant MIN_DELAY   =>  20;
use constant DELAY_RANGE =>  30;

sub lang_age_days;
sub report;
sub initialize_rows;
sub initialize_known_unknown;
sub gzb_word_length;

my $nxcgtx = undef;
my $roottypes = undef;

my $debug = 0; # 2;

my $sec = 0;
my $min = 0;
my $hour = 0;
my $mday = 0;
my $month = 0;
my $year = 0;
my $wday = 0;
my $yday = 0;
my $isdist = 0;

($sec, $min, $hour, $mday, $month, $year, $wday, $yday, $isdist) = localtime(time);
my $lang_age_years = ($year - 98);

my $starttime = time;

my $unknownfile = "unknown.rdb";
my $knownfile = "known.rdb";

my $slideshow_mode = 0;
my $wait_time = 3;
my $category_filter;
my $startuptest = 0;

GetOptions(
    'r|roottypes=s'  => \$roottypes,
    'd|debug' => \$debug,
    'c|category=s'	=> \$category_filter,
    's|slideshow' => \$slideshow_mode,
    'w|wait=f' => \$wait_time,
    'S|startuptest' => \$startuptest,
);

if ( defined $ARGV[0] ) {
    $nxcgtx = shift;
}

if ( not defined $nxcgtx ) {
    $nxcgtx = "nxcgtx.rdb";
}

if ( not defined $roottypes ) {
    $roottypes = "rdspnkm";
}


## Slurp in both unknown.rdb and known.rdb and nxcgtx.rdb, prepend a
## random number to each row of nxcgtx.rdb, and sort by the random
## number.  Use of various factors for multiplying the random numbers
## means definitely unknown words are more likely to show up early
## than words not yet quizzed, and words not yet quizzed are more
## likely to show up early than words definitely known.

my @rows;
my @fields;
my @unknownwords;
my @knownwords;

my %last_unknown_date;
my %last_known_date;
srand;

my $s = time;
&initialize_known_unknown;
&initialize_rows;

my $known = 0;
my $unknown = 0;
my $total = 0;

if ( not $slideshow_mode ) {
    print "dxiql-zox pxoq {h(eqnx)}, cxoq twax-txy kax-i hyw-tq-van henx,\n";
    print "rej pxoq {} cxoq te kax-i hyw-van, rej {gx(yl)} boq sru-van gxyl-zox.\n\n";
}

open UNKNOWN, ">> $unknownfile"  or die "can't open $unknownfile for writing";
open KNOWN, ">> $knownfile"  or die "can't open $knownfile for writing";

# flush unknown.rdb and known.rdb on every print
my $oldfh = select UNKNOWN;
$| = 1;		
select KNOWN;
$| = 1;
select $oldfh;

my $offset = -1;
my $line = "";
foreach $line ( @rows ) {
    ++$offset;
    #my $line = $_;
    @fields = split(  /\t/, $line );

    $total++;
    if ( $debug > 1 ) {
	print "\n" . $fields[0] . "\t\t" . $fields[ WORD ] . "\n";
    } else {
	#print "\n" . $fields[ WORD ] . "\n";
	print $offset . ": " if $debug;
	print $fields[ WORD ] . "\n";
    }
    
    if ( not $slideshow_mode ) {
	my $hyw = <STDIN>;
	if ( $hyw =~ /^h(e(nx)*)*/ ) {
	    $line =~ s/^[0-9\.]+\s+//;
	    $unknown++;
	    # re-quiz this unknown word later in this same session
	    my $requiz_delay = MIN_DELAY + (int (rand DELAY_RANGE));
	    print "requiz delay: $requiz_delay\n"  if $debug;
	    if ( $#rows < $offset + $requiz_delay ) {
		&initialize_rows;
	    } 
	    splice @rows, ($offset + $requiz_delay), 0, "00000000\t$line";
	    print UNKNOWN $fields[ WORD ] . "\t" . &lang_age_days . "\n";
	} elsif ( $hyw =~ /^g(x(y(l)*)*)*/) {
	    $total--;
	    if ( $total < 1 ) {
		close;
		exit;
	    }
	    &report;
	    close;
	    exit;
	} elsif ( $hyw =~ /^ *$/ ) {
	    $known++;
	    print KNOWN $fields[ WORD ] . "\t" . &lang_age_days  . "\n";
	} else { 
	    printf "\n?twax-noq-tq-zox. \n";
	}
    } else {
	select( undef, undef, undef, $wait_time);
	}
    
    print "\t" . $fields[ GLOSS ] . "\n";

    if ( $offset % 25 == 24 ) {
	&report;
    }
    
    if ( $slideshow_mode ) {
	my $shortsleep = (length $fields[ GLOSS ]) * ( $wait_time / 100 );
	select( undef, undef, undef, $shortsleep);
    }
}

close;

&report;
exit(0);

#==================================================

sub gzb_word_length {
    my $word = shift;
    $word =~ s/[-'xq]//g;
    return length( $word );
}


sub lang_age_days {
    ($sec, $min, $hour, $mday, $month, $year, $wday, $yday, $isdist) = localtime(time);

# gzb was started on or about 3 march 1998, i.e. on the 62nd day of
# that year.  This age calc is approx because it doesn't take leap
# days into account but I don't think I actually need such precision
# here.  The $year given by localtime() is years since 1900.

    my $lang_age_days = ($year - 98) * 365 + 62 + $yday;

    print "age of gzb in days: $lang_age_days \n" 	if $debug > 2;

    return $lang_age_days;
}


use constant BASE_WEIGHT => 100000;
use constant UNKNOWN_WEIGHT_PER_DAY => 1000;
use constant ARCHAIC_WEIGHT => 1000000;
use constant POSTP_WEIGHT => 60000;

sub initialize_rows {

    print "initializing @rows\n" if $debug;
    my @nxcgtx_rows;

    open NXCGTX,  $nxcgtx   or die "can't open $nxcgtx for reading";
    my $factor = 0;
    while ( <NXCGTX> ) {
	# first two lines are header, skip them
	if ( $. < 3 ) {
	    next;
	}
	chomp;
	s/__//g;
	my @init_fields = split(  /\t/, $_ );

	print "word = $init_fields[0] \t" 	if $debug;

	if ( $init_fields[ 1 ] !~ /[$roottypes]/ ) {
	    print qq(skipping because of undesired root type $init_fields[1]\n) if $debug;
	    next;
	} elsif ( defined $category_filter && $init_fields[ 3 ] !~ /$category_filter/ ) {
	    print qq(skipping because $init_fields[3] doesn't match category filter\n) if $debug;
	    next;
	}

	next if not defined $init_fields[2];

	my $factor = BASE_WEIGHT;
=pod	
	# weight more recently coined words more heavily than older ones
	my $date = $init_fields[DATE];
	# half the age of the language, a default weighting for undated words
	my $word_year = int(($year + 1900 - CREATION_YEAR) / 2) + CREATION_YEAR;
	if ( defined $date and $date =~ m/[0-9]+/ ) {
	    my @dateparts = split( /-/, $date );
	    $word_year = $dateparts[0];
	    $word_year =~ s/\?//;
	    print qq(got year $word_year from date string "$date"\n) 	if $debug;
	}

	my $word_age = $year+1900 - $word_year;
	my $divisor = ($lang_age_years * 2) - $word_age;
	print "\$year $year\t\$word_year = $word_year \t \$lang_age_years = $lang_age_years \t \$divisor = $divisor\n"	if $debug;
	my $factor = 10000 / $divisor;
=cut
	
	# don't match "archaically", as that would down-weight current words with 
	# outmoded senses, not just words that are entirely outmoded 

	# matching "archaic" at word boundary is still imperfect;
	# not all definitions with one or more current senses and one or more
	# archaic senses use "archaically" rather than "archaic"

	# why was I using quotemeta here?
    #    if ( defined $init_fields[2] and quotemeta( $init_fields[2] ) =~ /(archaic\b|nonce)/ ) {
	if ( $init_fields[2] =~ /(archaic\b|nonce)/ ) {
	    $factor = ARCHAIC_WEIGHT;
	}

	if ( $init_fields[1] eq "p" ) { # quiz more complex postpositions less often 
	    $factor = POSTP_WEIGHT * gzb_word_length( $init_fields[1] );
	}

	my $found_in_known =   grep { $init_fields[0] eq $_ } @knownwords ;
	my $found_in_unknown = grep { $init_fields[0] eq $_ } @unknownwords ;
	if ( $found_in_known and $found_in_unknown ) {
	    print qq("$init_fields[0]" found in both known and unknown files\n) if $debug;
	    my $datediff = $last_known_date{ $init_fields[0] } - $last_unknown_date{ $init_fields[0] };
	    if ( $datediff > 5 ) {
		$factor = BASE_WEIGHT; # 500000 / ( 1 + &lang_age_days - $last_known_date{ $init_fields[0] } );
	    } elsif ( $datediff > 0 ) {
		$factor = UNKNOWN_WEIGHT_PER_DAY * $datediff;
	    } else {
		$factor = BASE_WEIGHT;
	    }
	} elsif ( $found_in_known ) {
	    print qq("$init_fields[0]" found in known file\n) if $debug;
	    my $quizdate = $last_known_date{ $init_fields[0] } > 0  ?  $last_known_date{ $init_fields[0] }  :  &lang_age_days - 60;
	    $factor = BASE_WEIGHT #  / ( 1 + &lang_age_days - $quizdate );
	} elsif ( $found_in_unknown ) {
	    print qq("$init_fields[0]" found in unknown file\n) if $debug;
	    if ( $last_unknown_date{ $init_fields[0] } > 0 ) {
		# the longer it's been since we quizzed on this one, the
		# more likely we are to quiz in this session
		$factor =  UNKNOWN_WEIGHT_PER_DAY * ( &lang_age_days - $last_unknown_date{ $init_fields[0] } );
	    } else {
		$factor = UNKNOWN_WEIGHT_PER_DAY;
	    }
	}

	if ( $init_fields[1] eq "r" ) {
	    $factor /= 2;
	}

	my $word_row = sprintf "%08d\t%s", int(rand() * $factor), $_;
	print "\$word_row = [$word_row]\n"	if $debug;
	push @nxcgtx_rows, $word_row;
    }
    close NXCGTX;

    if ( $startuptest ) {
	print qq(exiting after startup because of startuptest option\n);
	exit;
    }

    push @rows, sort @nxcgtx_rows;

}


sub initialize_known_unknown {

    my @init_fields;
   if ( open UNKNOWN, "$unknownfile" ) {
       while ( <UNKNOWN> ) {
	   s/[\r\n]//g;
	   @init_fields = split(  /\t/, $_ );
	   push @unknownwords, $init_fields[0]; 
	   if ( defined $init_fields[1]  and  $init_fields[1] =~ /^[0-9]+$/ ) {
	       $last_unknown_date{ $init_fields[0] } = $init_fields[1];
	   } else {
	       $last_unknown_date{ $init_fields[0] } = 0;
	   }
       }
       close UNKNOWN;
   }

   if ( open KNOWN, "$knownfile" ) {
       while ( <KNOWN> ) {
	   s/[\r\n]//g;
	   @init_fields = split(  /\t/, $_ );
	   push @knownwords, $init_fields[0]; 
	   print "word $init_fields[0] is or was known.... " if $debug;
	   if ( defined $init_fields[1]  and  $init_fields[1] =~ /^[0-9]+$/ ) {
	       $last_known_date{ $init_fields[0] } = $init_fields[1];
	       print " storing $init_fields[1] as last known date\n" if $debug;
	   } else {
	       $last_known_date{ $init_fields[0] } = 0;
	       print " storing 0 as last known date\n" if $debug;
	   }
       }
       close KNOWN;
   }

}

sub report {
    return if $slideshow_mode;
    
    my $pct = ($known / $total) * 100;
    my $totaltime = time - $starttime;
    print "start time " . (strftime ('%Y-%m-%d %H:%M:%S', localtime($starttime))) . "\n" if $debug;
    my $days = int ( $totaltime / SECS_PER_DAY  );
    if ( $days ) {
	print "$days days, ";
    }
    my $hours = int ( ($days ? ($totaltime - ($days * SECS_PER_DAY)) : $totaltime ) / SECS_PER_HOUR );
    print "$hours hours, ";
    my $minutes = int (($totaltime % SECS_PER_HOUR ) / 60);
    print "$minutes minutes, ";
    print "$total total words, $known known, $unknown unknown, percent known = ";
    printf "%2.1f\n", $pct;
}
