#! /usr/bin/perl -w


#Twey recommends: <div style="display: inline-block;"><div>word</div><div>gloss</div></div>

# I'm trying <div class="ilc"> with a style element in the <HEAD> to reduce size of file

use strict;
use gzb_ascii_unicode;

use Getopt::Long;
Getopt::Long::Configure('bundling');


sub fix_html_entities( $ );

#$[ = 1;			# set array base to 1
$, = ' ';		# set output field separator
$\ = "\n";		# set output record separator

my $debug = 0;
my $lexicon = "nxcgtx.rdb";
my $convert_underscores_to_cite = 0;
my $convert_arrows = 0;

# by default, first para of doc becomes H1, other section openings are H2
# can make them H2 and H3 or whatever with this
my $top_heading_level = 1;
my $user_unkown_words_report;  	# undefined, will be set to "" if want default report filename or specified fname or left undefined if no unknown words report is wanted
my $help = 0;

my $rc = GetOptions(
    'l|lexicon=s'  => \$lexicon,
    'd|debug:i' => \$debug,		###TODO double-check I've remembered the optional argument syntax
    'h|header=i' => \$top_heading_level,
    'help' => \$help,
    '_|cite' 		=> \$convert_underscores_to_cite,
    'u|unknown:s' 	=> \$user_unkown_words_report,
    'a|arrows'		=> \$convert_arrows,
);

if ( $help ) {
    &usage;
    exit(0);
}

if ( not $rc ) { ###TODO handle this better
    die "GetOptions() returned 0";
}

my $sub_heading_level = $top_heading_level + 1;


# slurp in lexicon file, filtering out unwanted lines
open LEXICON, $lexicon  or die "can't open $lexicon for reading\n";
my @lexicon_words;
while ( <LEXICON> ) {
    my @f = split /\t/;
    if ( $f[1] =~ /^[bdfkmnprs]$/ ) {
	push @lexicon_words, $f[0];
    }
}
close LEXICON;



print '<html>';
print '<head>';
print '<title>';
# to be filled in manually by user
print 'glosses generated from ' . $ARGV[0];
print '</title>';

print qq(<meta name="generator" content="$0">);

# this line is because of the Internet Explorer bug where some unicode chars
# (e.g. IPA symbols) won't display unless the page stylesheet specifies a unicode font.
print '<style><!--';
print "BODY { font-family: \"Lucida Sans Unicode\", serif }";
print "DIV.ilc { display: inline-block; }";
print '--></style>';

print '<body>';


# paragraph types:
use constant PARA => 1;
use constant PRE => 2;
use constant TABLE => 3;
use constant HR => 4;
use constant H1 => 10;
use constant H2 => 11;
use constant UL => 12;
use constant OL => 13;	###TODO not implemented yet

# table types:
use constant INTERLINEAR => 1;
use constant DEFLIST => 2;
#DEFLIST = 1;
#INTERLINEAR = 2;


srand;
# suffixed to all element id attribute values so if output from multiple
# runs is pasted into one doc, all element id's will be unique
my $GID = int(rand(1) * 10000);

# status vars - where are we semantically/formatically in the doc
my $first = 1;
my $section = 0;
my $para_count = 0;
my $tag = 0;
my $last_para_type = 0;
#    my $parastart = 0;	# not sure if needed?
my $tablestart = -99;
my $in_para = 0;
my $tableline = 0;
my $tabletype = -1;
my $table_anchor = "";
my $fields_last_line = -1;
my $strong_open = 0;
my $cite_open = 0;

# hash of words found in text but not in lexicon; report 
# in case there are misspellings, or neologisms worth adding
# to lexicon
my %unknown;	###TODO add code to report on contents of this after finishing processing of input file

# for debugging
my $linkcount = 0;

while (<>) {
    s/[\r\n]+$//;

    my $line_before_html_entity_fixes = $_;
    $_ = &fix_html_entities( $_ );
    if ( $convert_arrows ) {
	s( &lt;- )( &#x2190; )g;
	s( -&gt; )( &#x2192; )g;
	s( &lt;--&gt; )( &#x2194; )g;
    }

    ###TODO: interact sanely with link: lines so underscores within
    ###URLs don't affect the $cite_open state but underscores in
    ###linked text do get proper cites
    if ( $convert_underscores_to_cite ) {
	while ( m/_/ ) {
	    if ( $cite_open ) {
		s(_)(</cite>);
		$cite_open = 0;
	    } else {
		s(_)(<cite>);
		$cite_open = 1;
	    }
	}
    }

    my @Fld = split(' ', $_, -1);

    if (/^\s*$/) {  	    # blank line
	if ($in_para) {
	    if ( $cite_open ) {
		print STDERR "line $.: unclosed underscore _ (<cite>) tag, closing it for you\n";
		print "</cite>";
		$cite_open = 0;
	    }
	    if ($tag == PARA) {
		print '</p>';
	    }
	    elsif ($tag == TABLE) {
		print '</table>';
	    }
	    elsif ($tag == PRE) {
		print "</pre>";
	    }
	    elsif ($tag == H1) {
		print "</H$top_heading_level>";
	    }
	    elsif ($tag == H2) {
		print "</H$sub_heading_level>";
	    }
	    elsif ($tag == UL) {
		print '</ul>';
	    }

	    if ($tabletype == INTERLINEAR) {
		print '</p></blockquote>';
		$tabletype = -1;
	    }
	}
	$last_para_type = $tag;
	$in_para = 0;
	$tableline = -1;
    }

    # on nonblank line
    if (!/^\s*$/) {
	# first line of new paragraph
	if (0 == $in_para) {
	    $in_para = 1;

	    $table_anchor = 't' . (++$para_count) . '_' . $GID;

	    # Tabs hint we need a table.
	    if ($_ =~ /\t+/) {
		print "<table cellpadding = \"10\" border=\"1\" id=\"" .

		$table_anchor . qq(">);
		$tabletype = DEFLIST;
		$tag = TABLE;
	    }
	    # three or more consecutive spaces hint we need a
	    # table (for interlinear glosses)
	    elsif ($_ =~ /   /) {
		print '<blockquote>';
		#print qq(<table cellspacing="10" id=") . $table_anchor . qq(">);
		print qq'<p>';
		$tag = PARA;
		$tabletype = INTERLINEAR;
		$tablestart = $.;

	    }
	    # a dashes line becomes a horizontal rule element
	    elsif ($_ =~ /^\s*----+\s*$/) {
		print '<hr>';
		$tag = HR;
	    }
	    elsif ($_ =~ /^=$/) {
		print "<pre>";
		s(=)();
		$tag = PRE;
	    }
	    # para with lines starting with '# ' become a list
	    ###TODO make this <ol> and use * or something for <ul>
	    elsif ($_ =~ /^\# /) {
		print '<ul>';
		$tag = UL;
	    }
	    else {
		if ($last_para_type == HR) {
		    print "<H$sub_heading_level id = \"section" . (++$section) . '_' . $GID . "\">";
		    $tag = H2;
		}
		elsif ($first) {
		    $first = 0;
		    print "<H$top_heading_level id = \"section" . (++$section) . '_' . $GID . "\">";
		    $tag = H1;
		}
		else {
		    print "<p id = \"p" . ++$para_count . '_' . $GID . "\">";
		    $tag = PARA;
		}
	    }
	}
    }

    # don't use the HTML-entities-replaced version of the line to get the URL from
    if ( $line_before_html_entity_fixes =~ m/^link: +([^\s]+)\s*(.*)/ ) {
	my $url = $1;
	my $linktext = &fix_html_entities( $2 );
	if ( $linktext eq '') {
	    $linktext = $url;
	}
	print qq(<a href="$url">$linktext</a>);
	s/.*//;	# blank out the line now that we've used its contents for a link,
	# so nothing will be printed later on.
    }

    if ( $in_para  &&  $tag != HR ) {
	s(--)( &mdash; )g;
    }

    if ( $in_para  &&  $tag == PARA  &&  $_ =~ /   / ) {
	print STDERR "warning: line $. has three or more " .
	    "spaces, suggesting an interlinear gloss, but we did not " . 
	    "make this a gloss due to first line of paragraph not " .
	    "having such spacing\n";
    }

    if ( $in_para && $tag == PARA && $tabletype == INTERLINEAR) {
	###TODO slurp the rest of the paragraph, use of inline-block style
	# requires treating the gzb sentence and the gloss line element
	# in pairs, not all one and the all the other...........

	print '<div class="ilc">';

	    # if first row of interlinear: glossed language text.
	    # Link each morpheme to dictionary if found there.
	    if ($. == $tablestart) {
		for (my $i = 0; $i <= $#Fld; $i++) {
		    ##### If you're modifying this for your own conlang
		    ##### you probably want to uncomment  the next line
		    ##### and comment out the remainder of this for loop.
		    ##### Or you could modify the other code as necessary
		    ##### to link to your own dictionary file.

		    #printf "<td> " . $Fld[ $i ] .  " </td> ";


		    ###TODO uncomment sometime when ready to finish
		    # implementing the feature that depends on this,
		    # back-linking from lexicon to sample sentences in
		    # grammar docs.  -- should write to a separate .rdb or
		    # .tab file rather than to STDERR?

		    #print STDERR $this_word . "\t" . $table_anchor;

		    # split element of gloss saving delimiters and word in alternate result fields
		    my @words = split(/([\s%!?.{}\[\]\(\),\"\*;:~]+)/, $Fld[$i] );
		    ### replace < > " & with HTML char entities -- where?
		    my $gloss_element = "";
		    for ( my $k = 0; $k < scalar(@words); $k++) {
			my $word = $words[$k];
			if ( grep { $word eq $_ } @lexicon_words ) {
			    $word = &make_hyperlink( $word );
			} elsif ( $word =~ /-/ ) {
			    if ( $word =~ /[A-Z][a-z]/ ) {
				$unknown{ $word } = 1;
			    }
			    # if no match found and word contains '-'
			    my @morphemes = split( /-/, $word );
			    my $linkedword = "";
			    my $j = 0;

			    for ( $j = 0; $j < scalar(@morphemes); $j++ ) {
				# check for match in nxcgtx array
				if ( grep { $morphemes[$j] eq $_ } @lexicon_words ) {
				    $linkedword .= &make_hyperlink( $morphemes[$j] );
				    ###TODO uncomment when finish implementing backlinks
				    #print STDERR $this_morpheme . "\t" . $table_anchor;
				    
				} else {
				    if ( $morphemes[$j] =~ /[A-Z][a-z]/ ) {
					$unknown{ $morphemes[$j] } = 1;
				    }
				    $linkedword .= $morphemes[$j];
				}
				if ( $j < (scalar(@morphemes) - 1) ) {
				    $linkedword .= "-";
				}
			    } ## end for each morpheme in compound
			    $word = $linkedword;
			} elsif ( $word =~ m/%%/ ) {
			    if ( $strong_open ) {
				$word =~ s(%%)(</strong>);
				$strong_open = 0;
			    } else {
				$word =~ s(%%)(<strong>);
				$strong_open = 1;
			    }
			} else {
			    if ( $word =~ /[A-Z][a-z]/ ) {
				$unknown{ $word } = 1;
			    }
			}
			$gloss_element .= $word;
		    } ## end for each chunk of punctuation or text within gloss-element
		    print &gzb_ascii_unicode_convert( $gloss_element ) . "</div>";
		} # end for each word on line

		$fields_last_line = $#Fld;
	    }
	    elsif ($. == $tablestart + 1) {
		while ( s/%% */<strong>/) {
		    s/ *%%/<\/strong>/;
		}

		for (my $i = 0; $i <= $#Fld; $i++) {
		    ###TODO handle these tags also within an ordinary paragraph 
		    # or deflist table
		    while ( $Fld[$i] =~ s(%% *)(<strong>) ) {
			$Fld[$i] =~ s( *%%)(</strong>);
		    }

		    print '<div> ' . $Fld[$i] . ' </div> ';
		}
		if ( $fields_last_line !=  $#Fld ) {
		    print STDERR "line $.: second line of gloss has different number of fields than first!";
		}
		$fields_last_line = $#Fld;
	    }
	    elsif ($. == $tablestart + 2) {
		my $colspan = $fields_last_line + 1;
		print qq(</div> <br> $_ \n);
	    }
	    else {
		print STDERR "line $.: warning: thought this para was interlinear gloss due to spacing, but it goes on >3 lines";
		print # qq(<td colspan="$fields_last_line"> $_ </td>);
	    }
	}



    if ($in_para && $tag == TABLE) {
	print '<tr>';
	###TODO: link words/morphemes in first column of table to lexicon entries
	s/\s+$//;
	s/\t+ */ \t<td> /g;
	my $line = $_;
	if ($_ !~ /^\<td/) {
	    $line = '<td> ' . $_;
	}
	print &gzb_ascii_unicode_convert( $line );
	

	print " </tr>\n";
    }
    elsif ($tag == HR) {
	# do nothing, we've already printed <hr> earlier
    }
    elsif ($tag == UL) {
	s/\#/<li>/;
	print &gzb_ascii_unicode_convert( $_ );
    }
    else {
	print &gzb_ascii_unicode_convert( $_ );
    }
} # end main while loop, on each line of input file

# final cleanup, close the open tag:
if ($in_para) {
    if ( $cite_open ) {
	print STDERR "line $.: unclosed underscore _ (<cite>) tag, closing it for you\n";
	print "</cite>";
    }

    if ($tag == PARA) {
	print '</p>';
    } elsif ($tag == TABLE) {
	print '</table>';
    } elsif ($tag == H1) {
	print '</h1>';
    } elsif ($tag == UL) {
	print '</ul>';
    } elsif ($tag == H2) {
	print '</h2>';
    } else {
	die "bad value for \$tag: $tag ";
    }
}

print "\n</body></html>";

###TODO write to a report file iff user says they want one
# rather than to stderr

if ( defined $user_unkown_words_report ) {
    ###TODO copy improved code for this from glossed doc framset scritp
    my @u = sort keys %unknown;
    if ( my $n = scalar @u ) {
	print STDERR "found $n words in glosses that are not in lexicon:";
	for ( my $i = 0; $i < $n;  $i++ ) {
	    print STDERR $u[ $i ];
	}
	print STDERR "\n";
    }
}

# ===

sub make_hyperlink( $ ) {
    $linkcount++;
    my $word = shift;
    my $anchor = $word;
    $anchor =~ s/\'//g;
    $anchor =~ s/x/__x/g;
    $anchor =~ s/q/__q/g;
    my $link = qq(<a href="n__xcgt__x.htm#$anchor" target="_blank">$word</a>);
    return $link;
}

sub fix_html_entities( $ ) {
    my $arg = shift;
    $arg =~ s(\&)(&amp;)g;
    $arg =~ s(\")(&quot;)g;
    $arg =~ s(<)(&lt;)g;
    $arg =~ s(>)(&gt;)g;
    return $arg;
}

sub usage {

print STDERR<<USAGE;
Usage: $0 <options>
-h N		base header level (defaults to 1)
--header=N
--help		This message
-_		Convert underscores to <cite> tags
--cite	
-a		Convert <- and -> to Unicode arrow dingbats
--arrows	
-l FILE		Lexicon input file.  Defaults to $lexicon
--lexicon=FILE
-u FILE		Write a report on unknown gzb words encounted in interlinear gloss
--unknown=FILE
USAGE

}
