#! /usr/bin/perl -w

# Convert gjax-zym-byn ASCII-transcription text from input file into
# Unicode HTML entities in output.  Some of the unicode mappings are
# not really ideal.

# (c) 2004-2011 Jim Henry.  Creative Commons licensing (not that it will
# be useful to anybody else as-is, but feel free to copy and modify)

#export gzb_ascii_unicode_convert;

$litero{'ax'} = "\&#226;";
$litero{'cq'} = "\&#269;";

$litero{'cx'} = "\&#265;";

#  litero["dx"] =  "\&#273;";  # d with crossbar, not very distinctive in Lucida Sans Unicode
$litero{'dx'} = "\&#240;";     # small edh
$litero{'eq'} = "\&#283;";

$litero{'fx'} = "\&#8355;";
# capital F with extra crossbar; franc symbol
$litero{'gx'} = "\&#285;";
$litero{'hq'} = "\&#295;";
# planck's constant sign
$litero{'hx'} = "\&#293;";

$litero{'iq'} = "\&#301;";
# i-breve.  There is no i-caron available.
$litero{'ix'} = "\&#238;";

#  litero["jq"] =  "\&#308;";      # capital J^ - not good
$litero{'jq'} = "\&#669;";
# j with crossed  tail (IPA for this sound)
$litero{'jx'} = "\&#309;";
$litero{'kq'} = "\&#1036;";
# cyrillic
$litero{'kx'} = "\&#311;";
# k-cedilla
$litero{'lq'} = "\&#321;";
# 'LATIN CAPITAL LETTER L WITH STROKE' 

$litero{'mq'} = "\&#625;";
# m with hook (is to m as eng is to n)
# IPA letter for voiced labiodental nasal

$litero{'nq'} = "\&#328;";
$litero{'nx'} = "\&#331;";

$litero{'ox'} = "\&#244;";

$litero{'oq'} = "\&#466;";

$litero{'px'} = "\&#934;";
# capital Phi
###  litero["pq"] =  "\&#664;";   # IPA bilabial click
$litero{'pq'} = "\&#421;";
# lowercase p-hook

$litero{'rx'} = "\&#x155;";# r-acute
$litero{'rq'} = "\&#345;";
# r-hacek

$litero{'sq'} = "\&#353;";
# s-hacek
$litero{'sx'} = "\&#349;";
# s-circumflex

$litero{'tq'} = "\&#357;";
# t-apostrophe
###  litero["tx"] =  "\&#8364;"; # euro sign
#  litero["tx"] =  "\&#359;"; # lowercase t with extra cross bar
$litero{'tx'} = "\&#x3B8;";
# lowercase theta

$litero{'ux'} = "\&#365;";
# u-breve, not used in gzb but some
# Esperanto text is included in gzb files

###  litero["vx"] = "\&#1509;";  # something from hebrew
$litero{'vx'} = "\&#436;";
# lowercase y-hook

$litero{'zq'} = "\&#382;";
# z-hacek 
$litero{'zx'} = "\&#378;";
# z-acute

#while (<>) {

sub gzb_ascii_unicode_convert {
    my $arg = shift;

    # mask English words so they are not affected
    $arg =~ s/que/_QU_e/g;
    $arg =~ s/qua/_QU_a/g;
    $arg =~ s/qui/_QU_i/g;
    $arg =~ s/quo/_QU_o/g;
    $arg =~ s/quu/_QU_u/g;
#    $arg =~ s/ffix/f__FFX__/g;
    # suffix, phoenix, infix, circumfix, ....
    $arg =~ s/[eu][fmn]ix/e__FFX__/g;
    $arg =~ s/ixe/__I_X__e/g;
    $arg =~ s/ixi/__I_X__i/g;
    $arg =~ s/Unix/Un__I_X__/g;

    # this one is tricky... occurs in {mixn'tiq, mix'hqax, vixl'pun...}
#   $arg =~ s/mix([^nwm'])/m__I_X__$1/g;
#    $arg =~ s/([bcdfghjklmnpqrstvwxz])ix([^nmwjrl'])/$1__I_X__$2/g;
    $arg =~ s/ix([^nmwjrl'])/__I_X__$1/g;
    $arg =~ s/ix$/__I_X__/g;
#    $arg =~ s/(.)ax([^jwrnm'-])/$1__A_X__$2/g;
#    $arg =~ s/([A-Za-z][^qxjwlr]*)ax([abcdefghiklopqstuvxyz])/$1__A_X__$2/g;
#    $arg =~ s/(.)ax$/$1__A_X__/g;
#    $arg =~ s/([A-Za-z][A-Za-z][jwlr]*)ax$/$1__A_X__/g;

    $arg =~ s/axe/__A_X__e/g;
    $arg =~ s/relax/rel__A_X__/g;
    $arg =~ s/oax/o__A_X__/g;
    $arg =~ s/([Gg]al)ax/$1__A_X__/g;
    $arg =~ s/axi/__A_X__i/g;
    # axis, axial, ...
    $arg =~ s/syntax/synt__A_X__/g;
    $arg =~ s/txt/_T_X_T_/g;
    # ASCII file extension
    $arg =~ s/pobox/pob__O_X__/g;
    # pobox.com, my PURL provider
    $arg =~ s/oxe/__O_X__e/g;
    #oxen,...
    $arg =~ s/oxi/__O_X__i/g;
    #oxidize....
    $arg =~ s/oxy/__O_X__y/g;
    #oxygen...

    $arg =~ s/([a-zA-Z]b)ox/$1__O_X__/g;

    $arg =~ s/uxi/__U_X__i/g;
    #auxiliary
    $arg =~ s/uxu/__U_X__u/g;
    # luxury
    $arg =~ s/anxi/a__N_X__i/g;
    # anxiety, anxious, ...

    # TO DO: make it sensitive to not do this conversion on quoted text
    # within tags <a href="" > etc ... will be tricky since such tags may
    # span lines

    foreach $xlit (keys %litero) {
	$arg =~ s/$xlit/$litero{$xlit}/g;
    }

    $arg =~ s/_QU_/qu/g;
    $arg =~ s/__FFX__/fix/g;
    $arg =~ s/__I_X__/ix/g;
    $arg =~ s/__A_X__/ax/g;
    $arg =~ s/__O_X__/ox/g;
    $arg =~ s/__U_X__/ux/g;
    $arg =~ s/_T_X_T_/txt/g;
    $arg =~ s/__N_X__/nx/g;

    $arg =~ s/__//g;

    return $arg;
}
