#! /usr/bin/perl -w

# take a mail-links text file and check if the URLs in it are already
# linked in one or more HTML file.

use strict;

use Getopt::Long;

#my $input = "";
my @text_files;
my @html_files;
my $output = "";
#my $html = "";

Getopt::Long::Configure('bundling');

GetOptions(
	't|text=s@' 	=> \@text_files,
    	'o|output=s' 	=> \$output,
	'h|html=s@'	=> \@html_files,
);

my $fatal = 0;
if ( (scalar @text_files ) == 0 ) {
    print "missing -t or --text option(s): one or more input text file containing hyperlinks\n";
    $fatal = 1;
}

if ( $output eq "" ) {
    print "missing -o or --output option: output file\n";
    $fatal = 1;
}

if ( (scalar @html_files) == 0 ) {
    print "missing -h or --html option(s): one or more html files to search for already used links\n";
    $fatal = 1;
}

exit(1)	 if $fatal;


my @html_contents;

foreach ( @html_files ) {
    open HTML, $_		or die "can't open $_ for reading\n";;
    while ( <HTML> ) {
	push @html_contents, $_;
    }
    close HTML;
}

if ( scalar  @html_contents < 1 ) {
    die "html files listed are empty\n";
}

open OUT, ">" . $output		or die "can't open $output for writing\n";;

foreach (  @text_files ) {
    open IN, $_		or die "can't open $_ for reading\n";
    while ( <IN> ) {
	my $line = $_;
	$line =~ s/[\r\n]+//g;
	my @words = split /\s+/;
	foreach ( @words ) {
	    my $word = $_;

	    ###TODO why so many false negatives?  could it be the
	    ###ampersands in the brown.edu links acting like regex
	    ###special chars?

	    if ( $word =~ m/^http:/ 
		 and grep (( index( $_, $word ) > -1 ), @html_contents ) ) 
	    {
		$line = "X\t" . $line;
	    }
	}
	print OUT $line . "\n";
    }
    close IN;
}

close OUT;
