#! /usr/bin/perl -w

use strict;


my $debug = 1;
my $inputfile = "nxcgtx.rdb";
my $outputfile = "nxcgtx_sorted.rdb";
my $sortkey = 4; # zero-based array offset
my $arg;

sub sorter {
    if ( not defined $$a{"sortkey"} ) {
	$$a{"sortkey"} = "";
    }

    if ( not defined $$b{"sortkey"} ) {
	$$b{"sortkey"} = "";
    }

    my $compare = $$a{"sortkey"} cmp $$b{"sortkey"};
    if ( $compare ) {
	return $compare;
    }

    my @fa = split /\t/, $$a{"row"};
    my @fb = split /\t/, $$b{"row"};
    
    return $fa[0] cmp $fb[0];    
}

while ( $arg = shift ) {
    print "processing \$arg == $arg\n" if $debug;
    if ( $arg eq "-d" ) {
	$debug = shift;
	print "setting \$debug to $debug\n"  if $debug;
	next;
    }

    if ( $arg eq "-i" ) {
	$inputfile = shift;
	next;
    }

    if ( $arg eq "-o" ) {
	$outputfile = shift;
    }

    if ( $arg eq "-f" ) {
	$sortkey = shift;
    }
}


my @header;
my @rows;

open IN, $inputfile 	or die "can't open $inputfile for reading\n";
open OUT, "> $outputfile" 	or die "can't open $outputfile for writing\n";

while ( <IN> )
{
    s/[\r\n]+//;
    if ( $. <= 2 ) {
	push @header, $_;
    } else {
	next if m/^\s*$/;
	my $row = $_;
	my @fields = split /\t/;
	my %rowpair;
	$rowpair{"sortkey"} = $fields[ $sortkey ];
	$rowpair{"row"} = $row;
	push @rows, \%rowpair;
    }
}

print OUT $header[0] . "\n";
print OUT $header[1] . "\n";


#foreach ( sort { $$a{"sortkey"} cmp $$b{"sortkey"} } @rows ) {
foreach ( sort sorter @rows ) {
    my %thisrowpair = %{ $_ };
    print OUT $thisrowpair{ "row" } . "\n";
}
