#! /usr/bin/perl -w

use strict;
use warnings;

my $fname = "link_examples.tab";
my $dict = "nxcgtx.rdb";
my $debug = 1;
my $in_count = 0;
my $out_count = 0;

my %sample_links;

my @nxcgtx_words;

open NXCGTX, $dict	or die "can't open $dict for reading\n";

while ( <NXCGTX> ) {
    my @f = split "\t", $_;
    if ( $f[1] =~ m/[bfmpksndr]/ ) {
	push @nxcgtx_words, $f[0];
    }
}
close NXCGTX;

print "slurped " . scalar @nxcgtx_words . " words \n";
#exit;

open LINKS, $fname	or die "can't open $fname for reading\n";

while ( <LINKS> ) {
    # fields: word, comma-separated list of sentence IDs
    chomp;
    next if m/^\s*$/;
    $in_count++;

    my @f = split m/\t/;
    if ( not defined $f[1]  or  $f[1] !~ m/^[0-9]+(,[0-9]+)*$/ ) {
	print STDERR "badly formatted sentence ID list for $f[0] \n";
	next;
    }

    if ( not grep { $_ eq $f[0] } @nxcgtx_words ) {
	print STDERR "$f[0] not found in $dict\n";
    }

    my @sentence_ids = split m/,/, $f[1];
    if ( not defined $sample_links{ $f[0] } ) {
	$sample_links{ $f[0] } = \@sentence_ids;
    } else {
	print STDERR "$f[0] has two or more sentence ID lists, need to merge them\n"	if $debug;
	push @{ $sample_links{ $f[0] } },  @sentence_ids;
    }
}

close LINKS;

if ( $debug ) {
	print STDERR "read $in_count sample sentence ID lists\n";
}

# rename old file to .bak
# output sorted list of words, each with sorted and uniqued list of IDs

rename $fname, $fname . ".bak"	or die "couldn't rename to $fname.bak\n";

open LINKS, ">$fname"		or die "can't open $fname for writing\n";

foreach ( sort keys %sample_links ) {
    print LINKS $_ . "\t";

    my @ids = @{ $sample_links{ $_ } };
    my %ids;
    foreach ( @ids ) {
	$ids{ $_ } = 1;
    }
    my @unique_ids = sort keys %ids;
    if ( scalar @unique_ids >= 5 ) {
	print STDERR "$_ has " . scalar @unique_ids . " sample sentences\n";
    }
    my $list = join ",", @unique_ids;
    print LINKS "$list\n";
    $out_count++;
}

close LINKS;

if ( $debug ) {
	print STDERR "wrote $out_count sample sentence ID lists\n";
}
